@@ -1,7 +1,9 @@
-import logging
+from cdislogging import get_logger
 import click
 import gen3.configure as config_tool
 
+logging = get_logger("__name__")
+
 
 @click.command()
 @click.option("--profile", help="name of the profile to name for this credentials")
