@@ -48,6 +48,9 @@ def get_or_create_event_loop_for_thread():
 
 
 def raise_for_status_and_print_error(response):
+    """
+    This only works for sync requests
+    """
     try:
         response.raise_for_status()
     except requests.HTTPError as exception:
@@ -269,7 +272,7 @@ DEFAULT_BACKOFF_SETTINGS = {
     "logger": None,
     "on_backoff": log_backoff_retry,
     "on_giveup": log_backoff_giveup,
-    "max_tries": os.environ.get("GEN3SDK_MAX_RETRIES", 3),
+    "max_tries": int(os.environ.get("GEN3SDK_MAX_RETRIES", 3)),
     "giveup": exception_do_not_retry,
 }
 
@@ -279,6 +282,6 @@ BACKOFF_NO_LOG_IF_NOT_RETRIED = {
     "logger": None,
     "on_backoff": log_backoff_retry,
     "on_giveup": log_backoff_giveup_except_on_no_retries,
-    "max_tries": os.environ.get("GEN3SDK_MAX_RETRIES", 3),
+    "max_tries": int(os.environ.get("GEN3SDK_MAX_RETRIES", 3)),
     "giveup": exception_do_not_retry,
 }
